/*
Barry Taylor's Better Volume Indicator as described at
http://emini-watch.com/free-stuff/volume-indicator/
Coded for NinjaTrader by Alex Matulich, 2008-08-18, Unicorn Research Corporation.
This indicator uses True Range rather than Range, it plots an exponential
moving average of the volume instead of simple moving average for efficiency,
and it optionally paints the price bars when BetterVolume indicates something
other than the normal color.

Climax volume - high volume and high range
High volume churn - high volume but low range
Climax volume plus High volume churn - both the above conditions (rare)
Low volume churn - low volume and high range
Low volume - just low volume but not a low volume churn bar
Watch for these at different stages of the market:

                  | Bottom | Start |Continue|  Top   |  Start  |Continue
                  |        |uptrend|uptrend |        |downtrend|downtrend
------------------+--------+-------+--------+--------+---------+---------
Volume Climax     |        |  Yes  |        |  Yes   |         |   Yes
High Volume Churn |  Yes   |       |        |  Yes   |         |   Yes 
Climax + Hi Churn |        |       |        |  Yes   |         |   Yes 
Low Volume Churn  |  Yes   |       |  Yes   |        |  Yes    |
Low Volume        |  Yes   |       |  Yes   |  Yes   |         |
-------------------------------------------------------------------------
*/
#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Better Volume indicator from emini-watch.com/free-stuff/volume-indicator/ using true range instead of range in calculations.
    /// </summary>
    [Description("Better Volume indicator from emini-watch.com/free-stuff/volume-indicator/ using true range instead of range in calculations.")]
    public class BetterVolume : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int lookback = 20; // Default setting for Lookback
			private bool paintbars = true; // Default setting for Paintbars
			private bool useTrueRange = true; // Default setting for UseTrueRange
        // User defined variables (add any user defined variables below)
            private int barwidth = 2;
			private DataSeries vxr, vdr;
			private double vwt;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			int hilight=barwidth+1;
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Silver), barwidth), PlotStyle.Bar, "VBar"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Goldenrod), hilight), PlotStyle.Bar, "LowBar"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Red), hilight), PlotStyle.Bar, "ClimaxBar"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Green), hilight), PlotStyle.Bar, "ChurnBar"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Magenta), hilight), PlotStyle.Bar, "ClimaxChurnBar"));
            Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Black), hilight), PlotStyle.Bar, "LowChurnBar"));
			Add(new Plot(new Pen(Color.FromKnownColor(KnownColor.Red), 1), PlotStyle.Line, "VolEMA"));

            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;

			vxr = new DataSeries(this);
			vdr = new DataSeries(this);
			vwt = 2.0 / (10.0 * lookback + 1.0); // bar average period is 10*lookback
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			double range, v = Volume[0];
			if ( CurrentBar < 1) {
				range = High[0] - Low[0];
				VolEMA.Set(v);
				return;
			}
			range = useTrueRange ?
				Math.Max(High[0], Close[1]) - Math.Min(Low[0], Close[1]) // use true range
				: High[0] - Low[0]; // use normal range
			VolEMA.Set(vwt * (v - VolEMA[1]) + VolEMA[1]); // faster than SMA

			int cc = 0; // climax churn detection
			int bc = 0; // bar color, default 0 (normal volume bar)

			vxr.Set(v * range);
			vdr.Set((range != 0.0) ? v / range : 0.0);
			if (v == MIN(Volume, lookback)[0]) bc = 1; //LowBar;
			if (vxr[0] == MAX(vxr, lookback)[0]) { bc = 2; ++cc; } //ClimaxBar
			if (vdr[0] == MAX(vdr, lookback)[0]) { bc = 3; ++cc; } //ChurnBar
			if (cc == 2) bc = 4; //ClimaxChurnBar;
			if (vdr[0] == MIN(vdr, lookback)[0]) bc = 5; //LowChurnBar
			
			Values[bc].Set(v); //plot whatever we found
			if (paintbars && bc > 0) BarColor = Plots[bc].Pen.Color;
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VBar
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LowBar
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ClimaxBar
        {
            get { return Values[2]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ChurnBar
        {
            get { return Values[3]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ClimaxChurnBar
        {
            get { return Values[4]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries LowChurnBar
        {
            get { return Values[5]; }
        }
		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VolEMA
        {
            get { return Values[6]; }
        }

        [Description("Volume bars to look back for min/max values")]
        [Category("Parameters")]
        public int Lookback
        {
            get { return lookback; }
            set { lookback = Math.Max(2, value); }
        }

		[Description("True=show paintbars, false=leave price bars alone.")]
        [Category("Parameters")]
        public bool Paintbars
        {
            get { return paintbars; }
            set { paintbars = value; }
        }

		[Description("True=use true range in calculations, false=use High-Low range.")]
        [Category("Parameters")]
        public bool UseTrueRange
        {
            get { return useTrueRange; }
            set { useTrueRange = value; }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private BetterVolume[] cacheBetterVolume = null;

        private static BetterVolume checkBetterVolume = new BetterVolume();

        /// <summary>
        /// Better Volume indicator from emini-watch.com/free-stuff/volume-indicator/ using true range instead of range in calculations.
        /// </summary>
        /// <returns></returns>
        public BetterVolume BetterVolume(int lookback, bool paintbars, bool useTrueRange)
        {
            return BetterVolume(Input, lookback, paintbars, useTrueRange);
        }

        /// <summary>
        /// Better Volume indicator from emini-watch.com/free-stuff/volume-indicator/ using true range instead of range in calculations.
        /// </summary>
        /// <returns></returns>
        public BetterVolume BetterVolume(Data.IDataSeries input, int lookback, bool paintbars, bool useTrueRange)
        {
            checkBetterVolume.Lookback = lookback;
            lookback = checkBetterVolume.Lookback;
            checkBetterVolume.Paintbars = paintbars;
            paintbars = checkBetterVolume.Paintbars;
            checkBetterVolume.UseTrueRange = useTrueRange;
            useTrueRange = checkBetterVolume.UseTrueRange;

            if (cacheBetterVolume != null)
                for (int idx = 0; idx < cacheBetterVolume.Length; idx++)
                    if (cacheBetterVolume[idx].Lookback == lookback && cacheBetterVolume[idx].Paintbars == paintbars && cacheBetterVolume[idx].UseTrueRange == useTrueRange && cacheBetterVolume[idx].EqualsInput(input))
                        return cacheBetterVolume[idx];

            BetterVolume indicator = new BetterVolume();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Lookback = lookback;
            indicator.Paintbars = paintbars;
            indicator.UseTrueRange = useTrueRange;
            indicator.SetUp();

            BetterVolume[] tmp = new BetterVolume[cacheBetterVolume == null ? 1 : cacheBetterVolume.Length + 1];
            if (cacheBetterVolume != null)
                cacheBetterVolume.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheBetterVolume = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Better Volume indicator from emini-watch.com/free-stuff/volume-indicator/ using true range instead of range in calculations.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BetterVolume BetterVolume(int lookback, bool paintbars, bool useTrueRange)
        {
            return _indicator.BetterVolume(Input, lookback, paintbars, useTrueRange);
        }

        /// <summary>
        /// Better Volume indicator from emini-watch.com/free-stuff/volume-indicator/ using true range instead of range in calculations.
        /// </summary>
        /// <returns></returns>
        public Indicator.BetterVolume BetterVolume(Data.IDataSeries input, int lookback, bool paintbars, bool useTrueRange)
        {
            return _indicator.BetterVolume(input, lookback, paintbars, useTrueRange);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Better Volume indicator from emini-watch.com/free-stuff/volume-indicator/ using true range instead of range in calculations.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BetterVolume BetterVolume(int lookback, bool paintbars, bool useTrueRange)
        {
            return _indicator.BetterVolume(Input, lookback, paintbars, useTrueRange);
        }

        /// <summary>
        /// Better Volume indicator from emini-watch.com/free-stuff/volume-indicator/ using true range instead of range in calculations.
        /// </summary>
        /// <returns></returns>
        public Indicator.BetterVolume BetterVolume(Data.IDataSeries input, int lookback, bool paintbars, bool useTrueRange)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.BetterVolume(input, lookback, paintbars, useTrueRange);
        }

    }
}
#endregion
