#region Using declarations
using System;
using System.IO;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

public enum ValueAreaType
{
	VOC,		
	TPO,
	VWTPO,
}

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    // <summary> (origonal -> CalculateValueArea)
    // The Value Area is the price range where 70% of yesterdays volume traded
	// Written by Ben L. at sbgtrading@yahoo.com
	//   Theory taken from several sources, but summarized at: http://www.secretsoftraders.com/ValueAreaHelpGuide.htm
	// May God bless you and your trading!
	//
	
	//	Description of the "ProfileType" parameter:

	//		I've given the option of creating the profile in 3 different ways:
	//			1)  VOC - This method loads all the volume of a bar onto the closing price of that bar.
	//						e.g.  A 5-minute bar has a volume of 280 and a range of 1.5 points with a close at 1534.25, then
	//						all 280 hits of volume are loaded onto the 1534.25 closing price.
	//			2)  TPO - This method disregards volume altogether, and gives a single hit to each price in the range of the bar.
	//						e.g.  A 5-minute bar has a range of 1.5 points with a High at 1534 and a Low at 1532.5, then
	//						1 contract (or "hit") is loaded to the seven prices in that range: 1532.50, 1532.75, 1533.0, 1533.25, 1533.50, 1533.75, and 1534
	//			3)  VWTPO - This method distribues the volume of a bar over the price range of the bar.
	//						e.g.  A 5-minute bar has a volume of 280 and a range of 1.5 points with a High at 1534 and a Low at 1532.5, then
	//						40 contracts (=280/7) are loaded to each of the seven prices in that range: 1532.50, 1532.75, 1533.0, 1533.25, 1533.50, 1533.75, and 1534
	//
	// Mods DeanV - 11/2008
	//	see also http://www.enthios.com/universal.htm
	//	restructured code.
	//	SetSessionTimes can return previous session ranges.
	//
	// </summary>
    [Description("The Value Area is the price range where 70% of yesterdays price / volume traded.  See Larry Levin www.secretsoftraders.com/ for more info")]
    public class dValueArea1 : Indicator
    {
        #region Variables
        // Wizard generated variables
			private bool inclWeekendVol=false;
			private int openHour=0,openMinute=0;
			private double pctOfVolumeInVA = 0.70;
			private ValueAreaType profileType=ValueAreaType.VWTPO;
			private double sessionLengthInHours=23;
			private int totalSlots= 100;
			private int presentMethod= 1;	//0=no mapping (best for real time), 1,2 = too busy for real time.
		// User defined variables (add any user defined variables below)
			private double[,] PriceHitsArray;// = new double[2,500];
			private double TheSessionHigh,TheSessionLow;
			private double VAtop=0.0,VAbot=0.0,PriceOfPOC=0.0;//,HitsTotal=0.0;
			private DateTime CSEndTime,CSStartTime;
		
		private IntSeries SesNum;
		private int sEndBnum = 0;
		private int sStartBnum = 0;	//first date bar# we care about
 		private SolidBrush		SessBrush, PreSessBrush;
		private double sessVAtop=0.0,sessVAbot=0.0,sessPriceOfPOC=0.0,sessMaxHits=0.0;
		
		//make these global and set on plot call, so we don't have to pass around.
		private Rectangle PlotBounds;
		private double PlotMin, PlotMax;
		private double PlotMMDiff;
		private int lastBar;	//right most bar#.
		private int firstBar;	//left most bar# in display

		private int TicksPerSlot;
		private int LastSlotUsed;
		private double SlotHalfRange;
		
		
		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Orange, PlotStyle.Dot, "VAt"));
            Add(new Plot(Color.Pink, PlotStyle.Dot, "VAb"));
            Add(new Plot(Color.Purple, PlotStyle.Line, "POC"));
			AutoScale			= false;
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
			
			PriceHitsArray = new double[2,totalSlots];
			SesNum = new IntSeries(this);
			CalcColors();
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {			
			double Variance=0.0;
			double StandardDeviation=0.0;

			if(CurrentBar<10) {
				SetCurSessEndTime();	//get it started.
				SesNum.Set(0);
				return;
			}

			SesNum.Set(SesNum[1]);	//copy previous # for default.
			
			// (do on first (complete) bar, outside of session range. Show previous session.)
			if( (Time[1].CompareTo(CSEndTime)==0)
				|| (Time[2].CompareTo(CSEndTime)<0 && Time[1].CompareTo(CSEndTime)>0)
				//and to catch only open hrs. charts, that don't end exactly as our paramaters...
				|| (Bars.SessionBreak && Time[1].CompareTo(CSEndTime)<0 && Time[0].CompareTo(CSEndTime)>0)
				) { NewSessStart();
			}
			
					
			if(VAtop>0.0)
			{	VAt.Set(VAtop);
				VAb.Set(VAbot);
				POC.Set(PriceOfPOC);
			}
			SetCurSessEndTime();	//get current session End date/time, so we can compare on next bar, regardless.

			}
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{ 	//Gets here anytime a screen draw/redraw range is requested.
			// Default plotting in base class. This indicator has some, so call it.
			base.Plot(graphics, bounds, min, max);
			if(presentMethod < 1) return;
			
			int x;
			
			int bars = Math.Min(ChartControl.LastBarPainted, ChartControl.BarsPainted);	//max bars that "could?" be in display.
			//int barPaintWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);

			lastBar		= Math.Min(ChartControl.LastBarPainted, Bars.Count - 1);	//right most bar#.
			firstBar	= (lastBar - bars) + 1;

			int leftMostSessNum = SesNum[(CurrentBar - firstBar + 1)];	//+1 for CurrentBar vs. lastbar.
			int rightMostSessNum = Math.Min(SesNum[(lastBar > CurrentBar ? 0 :CurrentBar - lastBar +1)],SesNum[0]);	//avoid uncompleted last bar > CurB.
			
			if (leftMostSessNum <= SesNum[0]) {
				//some previous session is in the display.
				//set these globals 1x, before any calls that use them
				PlotBounds = bounds;
				PlotMin = min;
				PlotMax = max;
				PlotMMDiff = ChartControl.MaxMinusMin(PlotMax,PlotMin); //don't know why we do it this way?
				
				// Find highest and lowest price points
				double ChartLowPrice  = double.MaxValue;
				for (int i = firstBar; i <= lastBar && i >= 0; i++) {
					ChartLowPrice  = Math.Min(ChartLowPrice , Bars.Get(i).Low);
				}
				int ScreenVLowPos = GetYPos(ChartLowPrice);	//rightmost low screen value

				//loop through screen dipslay day history				
				for(x = leftMostSessNum; x <= rightMostSessNum;x++) {
					SetSessionBars(SesNum[0] - x);
					if(sEndBnum > sStartBnum) {	//regular session should be OK.
						DetermineHL_BarRange();	//using previous call.
					
						if(!StuffHits_BarRange()) return;
						
						//calc slot height for this BarRange.
						int SlotVHeight = ScreenVLowPos - GetYPos(ChartLowPrice + (SlotHalfRange*2.0));//-1;	//at least 1
						if(SlotVHeight < 1) SlotVHeight = 1;
						
						DrawSlots(graphics, SlotVHeight, x, SessBrush);
					}
					if(x == SesNum[0]) {
						//Some Text...
						int bago = Math.Max((CurrentBar - lastBar)+2, 0);

						DrawText("sessVAtop",(sessVAtop).ToString("0.0000"),bago,sessVAtop,Color.Orange);		
						DrawText("sessVAbot",(sessVAbot).ToString("0.0000"),bago,sessVAbot,Color.Pink);		
						DrawText("sessPriceOfPOC",(sessPriceOfPOC).ToString("0.0000"),bago,sessPriceOfPOC,Color.Purple);		
					}
					
					if(presentMethod == 2) {
						SetPreSessBars(SesNum[0] - x);
						if(sEndBnum <= sStartBnum) continue;	//1st bar of presession, or presess hasn't started yet.

						DetermineHL_BarRange();	//using previous call.
						if(!StuffHits_BarRange()) continue;
						
						int SlotVHeight = ScreenVLowPos - GetYPos(ChartLowPrice + (SlotHalfRange*2.0));//-1;	//at least 1
						if(SlotVHeight < 1) SlotVHeight = 1;
					
						DrawSlots(graphics, SlotVHeight, x, PreSessBrush);
						
					}
				
				}
				
			}
		
		}

		private void NewSessStart() {
			//it's a new session, increment and calc POC from yesterday.
			if(!inclWeekendVol && (Time[1].DayOfWeek==DayOfWeek.Saturday || Time[1].DayOfWeek==DayOfWeek.Sunday)) return;
			//increment session number.
			//SesNum's switch on first bar after last bar of defined session.
			//  They include any pre / post bars displayed, as well as open hours.
			SesNum.Set(SesNum[0] + 1);
			SesNum.Set(1,SesNum[0]);	//one bar off.
			SetSessionBars(1);	//calc for just completed session
		
			DetermineHL_BarRange();	//using previous call.
			if(!StuffHits_BarRange()) return;
			
			VAtop = sessVAtop;
			VAbot = sessVAbot;
			PriceOfPOC = sessPriceOfPOC;
			
		}
		
		
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VAt
        {
            get { return Values[0]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries VAb
        {
            get { return Values[1]; }
        }
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries POC
        {
            get { return Values[2]; }
        }

        [Description("Include Weekend Volume")]
        [Category("Parameters")]
        public bool InclWeekendVol
        {
            get { return inclWeekendVol; }
			set { inclWeekendVol = value;}
        }
        [Description("Type of profile VOC,TPO,VWTPO")]
        [Category("Parameters")]
        public ValueAreaType ProfileType
        {
            get { return profileType; }
			set { profileType = value; }
//			set { profileType = value.ToUpper();
//				  if(profileType!="VOC" && profileType!="TPO" && profileType!="VWTPO") profileType="VWTPO";
//				}
        }
        [Description("Percent of volume within Value Area")]
        [Category("Parameters")]
        public double PctOfVolumeInVA
        {
            get { return pctOfVolumeInVA; }
			set { pctOfVolumeInVA = Math.Max(0.01,Math.Min(1.0,value));}
        }
        [Description("Market open hour IN 24HR FORMAT")]
        [Category("Parameters")]
        public int OpenHour
        {
            get { return openHour; }
			set { openHour = Math.Max(0,value);}
        }
        [Description("Market open minute")]
        [Category("Parameters")]
        public int OpenMinute
        {
            get { return openMinute; }
			set { openMinute = Math.Max(0,value);}
        }
        [Description("Session length (in hours)")]
        [Category("Parameters")]
        public double SessionLengthInHours
        {
            get { return sessionLengthInHours; }
			set { sessionLengthInHours = value;}
        }
        [Description("Total Slots to Calc")]
        [Category("Parameters")]
        public int TotalSlots
        {
            get { return totalSlots; }
			set { totalSlots = Math.Max(3,Math.Min(500,value));}
        }
        [Description("How to (show) value maps. 0=no mapping (best for real time), 1,2 = too busy for real time.")]
        [Category("Parameters")]
        public int PresentMethod
        {
            get { return presentMethod; }
			set { presentMethod = Math.Max(0,Math.Min(5,value));}
        }
        #endregion
//===================================================================================

//================ DeanV mod/add stuff below.
		
		private void SetPreSessBars(int SessionAgo) {
			int sNum = SesNum[0] - SessionAgo;
			//have to have a default. Usefull if sessionago == 0.
			//this is set to Real sess start.
			DateTime StartTime = new DateTime(Time[0].Year,Time[0].Month,Time[0].Day,OpenHour,OpenMinute,0,0,DateTimeKind.Utc);
			
			sEndBnum = sStartBnum = 0;
			if(sNum == SesNum[0] || sNum < 0) {	//current session
				for(int x = 1 ; x < CurrentBar; x++) {	//start at newest barago and go backwards
					//make sure we set the endbar to just before real sess start.
					if(sEndBnum == 0 && Time[x].CompareTo(StartTime) <= 0) {	// 1 bar too far.
						sEndBnum = CurrentBar - x;// +1;	//pre-session bars end here.
					}
					if(SesNum[x] != SesNum[0]) { //first time this happens, it's session start.
						sStartBnum = CurrentBar - x +2;	//pre-session bars starts here.
						break;	//should be done.
					}
				}
				if(sEndBnum < sStartBnum) {
					if(sEndBnum == 0) sEndBnum = CurrentBar;
					if(sStartBnum == 0) sStartBnum = 1;
				}

			}
			else { //find and set pervious session date/times and barnumber range we care about.
				for(int x = 1 ; x < CurrentBar; x++) {	//start at newest barago and go backwards to catch right date
					if(sEndBnum == 0 && SesNum[x] == sNum) {	// last bar of session range
						StartTime = new DateTime(Time[x].Year,Time[x].Month,Time[x].Day,OpenHour,OpenMinute,0,0,DateTimeKind.Utc);
						if(Time[x].CompareTo(StartTime) <= 0) 	// 1 bar too far.
							sEndBnum = CurrentBar - x;// +1;
					}
					if(SesNum[x] < sNum) {	// 1 bar too far.
						sStartBnum = CurrentBar - x +2;	//pre-session start bars.
						break;	//end
					}
				}
				if(sEndBnum < sStartBnum) {
					sEndBnum = sStartBnum = 1;
				}
			}
		}
	
		private void SetSessionBars(int SessionAgo) {
			//have to have a default. Usefull if sessionago == 0.
			DateTime StartTime = new DateTime(Time[0].Year,Time[0].Month,Time[0].Day,OpenHour,OpenMinute,0,0,DateTimeKind.Utc);
			int sNum = SesNum[0] - SessionAgo;
		
			if(sNum == SesNum[0] || sNum < 0) {	//not a previous session		
				sEndBnum = sStartBnum = CurrentBar;
				for(int x = 1 ; x < CurrentBar; x++) {	//start at newest barago and go backwards
					if(Time[x].CompareTo(StartTime) <= 0) {	// 1 bar too far.
						sStartBnum = CurrentBar - x +1;	//don't need pre-session bars.
						break;	//end
					}
				if(SesNum[x] != SesNum[0]) break;	//ses hasn't started yet.
				}

			}
			else { //find and set pervious session date/times and barnumber range we care about.
				sStartBnum = 1;
				sEndBnum = 0;
				for(int x = 1 ; x < CurrentBar; x++) {	//start at newest barago and go backwards to catch right date
					if(sEndBnum == 0 && SesNum[x] == sNum) {	// last bar of session range
						//skip weekends? (Weekends are included with Mondays sess# when skipped)
						sEndBnum = CurrentBar - x +1;
						//use last bar of session count to set correct date/times of session.
						StartTime = new DateTime(Time[x].Year,Time[x].Month,Time[x].Day,OpenHour,OpenMinute,0,0,DateTimeKind.Utc);
					}
					if(sEndBnum > 0 && Time[x].CompareTo(StartTime) <= 0) {	// 1 bar too far.
						sStartBnum = CurrentBar - x +1;	//don't need pre-session bars.
						break;	//end
					}
				}
				if(sEndBnum == 0) sEndBnum = CurrentBar;	//safty if havn't got there yet.
			}
		}

			
		private void DetermineHL_BarRange() {
			// use bar number range from SetSessionTimes(>0).
			int sBarsAgo= CurrentBar - sStartBnum; //first bar to check in barsago.
			int eBarsAgo= CurrentBar - sEndBnum; //last bar to check in barsago.
			
			TheSessionHigh = High[eBarsAgo];
			TheSessionLow = Low[eBarsAgo];
						
			for(int x = eBarsAgo ; x <= sBarsAgo; x++) {
				if(High[x] > TheSessionHigh) TheSessionHigh =High[x];
				if(Low[x] < TheSessionLow)   TheSessionLow  =Low[x];
			}			
		}

		private bool StuffHits_BarRange() {
			int x;
			
			int TicksInRange = (int) Math.Round((TheSessionHigh-TheSessionLow)/TickSize,0);
			
			//fit ticks into array by so many TicksPerSlot
			TicksPerSlot = (TicksInRange / totalSlots) +1;	//should just drop the fract part.
			LastSlotUsed = TicksInRange / TicksPerSlot;	//Zero based, drop fract part.
			SlotHalfRange = ((TickSize * TicksPerSlot) - TickSize) /2;

			//clear counts in any case.
			for(x=0;x<=LastSlotUsed;x++) {	// 0 -> 999, reset from bottom up.
				 	PriceHitsArray[0,x]=(x*TickSize*TicksPerSlot) + SlotHalfRange; //Lowest Tick Value/Slot upped to mid value point
				 	PriceHitsArray[0,x] += TheSessionLow; //add it to the bottom
					PriceHitsArray[1,x]=0.0;	//clear counts per value.
			}
			
			if( TicksInRange > 0) {	//do it
				
				int sBarsAgo= CurrentBar - sStartBnum; //first bar to check in barsago.
				int eBarsAgo= CurrentBar - sEndBnum; //last bar to check in barsago.
				
				int index=0;
				int i=eBarsAgo;
				double BarH;
				double BarL;
				while (i <= sBarsAgo) { //Accumulate the volume/hits for each previous days bar into PriceVolume array

					if(profileType == ValueAreaType.VOC) {//Volume On Close - puts all the volume for that bar on the close price
						index = (int) Math.Round((Close[i]-TheSessionLow)/TickSize,0);
						index /= TicksPerSlot;	//drop fract part.
						PriceHitsArray[1,index] += Volume[i];
					}
					if (profileType == ValueAreaType.TPO) {//Time Price Opportunity - disregards volume, only counts number of times prices are touched
						BarH=High[i]; BarL=Low[i];
						while(BarL <= BarH) {
							index = (int) Math.Round((BarL-TheSessionLow)/TickSize,0);	//ticks from bottom
							index /= TicksPerSlot;	//drop fract part.
							PriceHitsArray[1,index] += 1;	//increment this value count.
							BarL = BarL + TickSize;	//up 1 tick
						}
					}
					if (profileType == ValueAreaType.VWTPO) {//Volume Weighted Time Price Opportunity - Disperses the Volume of the bar over the range of the bar so each price touched is weighted with volume
						BarH=High[i]; BarL=Low[i];
						int TicksInBar = (int) Math.Round((BarH-Low[i])/TickSize+1,0);
						while(BarL<= BarH) {
							index = (int) Math.Round((BarL-TheSessionLow)/TickSize,0);
							index /= TicksPerSlot;	//drop fract part.
							PriceHitsArray[1,index] += Volume[i]/TicksInBar;
							BarL = BarL + TickSize;
						}
					}
					i++;
				}
				//arrays are stuffed.
				
				//Calculate the Average price as weighted by the hit counts AND find the price with the highest hits (POC price)
				i=0;
				double THxP=0.0; //Total of Hits multiplied by Price at that volume
				double HitsTotal=0.0;
				sessPriceOfPOC=0.0;
				sessMaxHits = 0.0;
				
				double midPoint = TheSessionLow + ((TheSessionHigh-TheSessionLow) *.5);
				double midUpCnt=0,midDnCnt=0;	//counts above/below midpoint.
				
				while(i<=LastSlotUsed) {	//Sum up Volume*Price in THxP...and sum up Volume in VolumeTotal
					if(PriceHitsArray[1,i]>0.0) {
						THxP += (PriceHitsArray[1,i] * PriceHitsArray[0,i]);
						HitsTotal += PriceHitsArray[1,i];
						if(PriceHitsArray[1,i] > sessMaxHits) { //used to determine POC level
							sessMaxHits = PriceHitsArray[1,i]; 
							sessPriceOfPOC = PriceHitsArray[0,i];
						}
						//sum up hits for possable later use
						if(PriceHitsArray[0,i] > midPoint) midUpCnt += PriceHitsArray[1,i];	//don't count equals
						if(PriceHitsArray[0,i] < midPoint) midDnCnt += PriceHitsArray[1,i];
					}
					i++;
				}
				//now lowest (or only) sessMaxHits/POC is known.
				//determine in others match, and pick best choice.
				//
				//Rules to use are:
				// 1. If there is more than 1 price with the same 'most' TPO's then the price closest to the mid-point of the range (high - low) is used. 
				// 2. If the 2 'most' TPO prices are equi-distance from the mid-point then the price on the side of the mid-point with the most TPO's is used. 
				// 3. If there are equal number of TPO's on each side then the lower price is used. 
				//
				int POCcount = 0;
				double mid1=midPoint,mid2=midPoint;	//Distance from midPoint, set larger than possable
				int mid1Dx = 0, mid2Dx = 0;	//array index count of finds.
				
				i=0;
				while(i<=LastSlotUsed) {	//scan known array from bottom to top
					if(PriceHitsArray[1,i] == sessMaxHits) {	//should be at least 1.
						POCcount++;
						//find 2 closest to midpoint
						if(Math.Abs(midPoint - PriceHitsArray[0,i]) <= mid1) {
							mid2 = mid1;//rotate next closest
							mid2Dx = mid1Dx;
							mid1 = Math.Abs(midPoint - PriceHitsArray[0,i]);	//how far away from midpoint
							mid1Dx = i;
						}
					}
					i++;
				}
				if(POCcount > 1) {
					if(mid1 != mid2) {	//found it, rule #1
						sessPriceOfPOC = PriceHitsArray[0,mid1Dx];
					}
					else {	//they are equal, Rule #2 may apply
						if(midUpCnt == midDnCnt) {	//must use Rule #3
							sessPriceOfPOC = PriceHitsArray[0,mid2Dx];	//use the lower.
						}
						else {	//Rule #2
							if(midUpCnt > midDnCnt) {
								sessPriceOfPOC = PriceHitsArray[0,mid1Dx];	//mid1 = upper of 2
							}
							else {
								sessPriceOfPOC = PriceHitsArray[0,mid2Dx];	//must be the lower.
							}
						}//end Rule #2
					}
				}
				//end of finding best fit for POC
				
				
				//AvgPrice = THxP/HitsTotal;
				sessVAtop=THxP/HitsTotal;
				sessVAbot=sessVAtop;

				 //This loop calculates the percentage of hits contained within the Value Area
				double ViA=0.0;
				double TV=0.00001;
				double Adj=0.0;
				while(ViA/TV < pctOfVolumeInVA) {
					sessVAbot = sessVAbot - Adj;
					sessVAtop = sessVAtop + Adj;
					ViA=0.0;
					TV=0.00001;
					for(i=0;i<=LastSlotUsed;i++) {
						if(PriceHitsArray[0,i] > sessVAbot-Adj && PriceHitsArray[0,i] < sessVAtop+Adj) ViA += PriceHitsArray[1,i];
						TV += PriceHitsArray[1,i];
					}
					Adj=TickSize;
				}
			}	//ticksinRange
			return true;
		}
		
		private void SetCurSessEndTime() {
			CSStartTime = new DateTime(Time[0].Year,Time[0].Month,Time[0].Day,OpenHour,OpenMinute,0,0,DateTimeKind.Utc);
			if(sessionLengthInHours>=24) CSEndTime = CSStartTime.AddHours(24.0-1.0/60.0);
			else CSEndTime = CSStartTime.AddHours(sessionLengthInHours);
		}


//======== graphic stuff ==========
		private void CalcColors() 
		{
			int				transparency	= 80 ;

			int	alpha	 = (int) (255.0 * ((100.0 - transparency) / 100.0));
		 	Color			sessColor		= Color.Lime;
		 	Color			presessColor		= Color.Red;

			SessBrush   = new SolidBrush(Color.FromArgb(alpha, sessColor.R, sessColor.G, sessColor.B));
			PreSessBrush     = new SolidBrush(Color.FromArgb(alpha, presessColor.R, presessColor.G, presessColor.B));
		}

		//this version uses per-set globals from Plot().
		private int GetYPos(double price) {
			int TotalHeight = PlotBounds.Y + PlotBounds.Height;
			
			int ret =  (int) ((TotalHeight) - ((price - PlotMin) / PlotMMDiff) * PlotBounds.Height);
			return ret;
		}
		
		private void DrawSlots(Graphics graphics, int slotHeight, int ThisSesNum, SolidBrush theBrush) {
			int x,yPos,vPos,vHeight;
			double price;
			int HWidth ;//  = (int) ((PlotBounds.Width * .5));	//1/2 screen
		
			//must calc left position from right edge for it to work right.
			int barPaintWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			int bars = Math.Min(ChartControl.LastBarPainted, ChartControl.BarsPainted);	//max bars in display.
			int leftMostPos = (((ChartControl.CanvasRight - ChartControl.BarMarginRight) - barPaintWidth) - ((bars -1) * ChartControl.BarSpace)) +1;// + (1 * ChartControl.BarSpace);

			int leftMostBarAgo = CurrentBar - firstBar + 1;	//firstBar is leftmost bar#
			
			int barsInRange = 0;
			for(x=0;x<bars;x++) {	//scan left to right
				if(SesNum[leftMostBarAgo-x] == ThisSesNum) {
					
					if(presentMethod > 0 && leftMostBarAgo-x > CurrentBar - sStartBnum +1) continue; //wait till we get an open session bar.
					if(presentMethod ==2 && leftMostBarAgo-x < CurrentBar - sEndBnum +1) continue; //stop wen we get an open session bar.
					barsInRange++;
				}
				if(barsInRange ==1 && x != 0) 	//reset leftPos since not really leftmost for most sess#.
					leftMostPos = ( ((ChartControl.CanvasRight - ChartControl.BarMarginRight) - barPaintWidth) - ((bars - 1) * ChartControl.BarSpace) ) 
					+ ( x * ChartControl.BarSpace) +1;
			}
			if(barsInRange-- < 2) return;	//have at least 1 bar to paint. Don't paint last bar.
			
			int rightMostPos = leftMostPos + (barsInRange * ChartControl.BarSpace);
			if( rightMostPos > PlotBounds.Width ) rightMostPos = PlotBounds.Width;
			
			int TotalHeight = PlotBounds.Y + PlotBounds.Height;
			for(x=0;x<=LastSlotUsed;x++) {
				price = PriceHitsArray[0,x] + SlotHalfRange;	//take it from mid to top.
				HWidth   = (int) ((rightMostPos - leftMostPos) * (PriceHitsArray[1,x] / ( sessMaxHits )));
				
				yPos =  (int) ((TotalHeight) - ((price - PlotMin) / PlotMMDiff) * PlotBounds.Height);
				vPos = yPos + slotHeight;
				if (yPos >= TotalHeight || vPos <= 1) continue;	//too low or high to display
				
				//take out any negitive portion
				yPos += (yPos < 0 ? yPos*-1 : 0);
				
				if(vPos > TotalHeight) vPos = TotalHeight;
				vHeight = vPos - yPos;
	
				graphics.FillRectangle(theBrush, new Rectangle(leftMostPos, yPos, HWidth, vHeight));
			}
			
		}

//===================================================================================

	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private dValueArea1[] cachedValueArea1 = null;

        private static dValueArea1 checkdValueArea1 = new dValueArea1();

        /// <summary>
        /// The Value Area is the price range where 70% of yesterdays price / volume traded.  See Larry Levin www.secretsoftraders.com/ for more info
        /// </summary>
        /// <returns></returns>
        public dValueArea1 dValueArea1(bool inclWeekendVol, int openHour, int openMinute, double pctOfVolumeInVA, int presentMethod, ValueAreaType profileType, double sessionLengthInHours, int totalSlots)
        {
            return dValueArea1(Input, inclWeekendVol, openHour, openMinute, pctOfVolumeInVA, presentMethod, profileType, sessionLengthInHours, totalSlots);
        }

        /// <summary>
        /// The Value Area is the price range where 70% of yesterdays price / volume traded.  See Larry Levin www.secretsoftraders.com/ for more info
        /// </summary>
        /// <returns></returns>
        public dValueArea1 dValueArea1(Data.IDataSeries input, bool inclWeekendVol, int openHour, int openMinute, double pctOfVolumeInVA, int presentMethod, ValueAreaType profileType, double sessionLengthInHours, int totalSlots)
        {
            checkdValueArea1.InclWeekendVol = inclWeekendVol;
            inclWeekendVol = checkdValueArea1.InclWeekendVol;
            checkdValueArea1.OpenHour = openHour;
            openHour = checkdValueArea1.OpenHour;
            checkdValueArea1.OpenMinute = openMinute;
            openMinute = checkdValueArea1.OpenMinute;
            checkdValueArea1.PctOfVolumeInVA = pctOfVolumeInVA;
            pctOfVolumeInVA = checkdValueArea1.PctOfVolumeInVA;
            checkdValueArea1.PresentMethod = presentMethod;
            presentMethod = checkdValueArea1.PresentMethod;
            checkdValueArea1.ProfileType = profileType;
            profileType = checkdValueArea1.ProfileType;
            checkdValueArea1.SessionLengthInHours = sessionLengthInHours;
            sessionLengthInHours = checkdValueArea1.SessionLengthInHours;
            checkdValueArea1.TotalSlots = totalSlots;
            totalSlots = checkdValueArea1.TotalSlots;

            if (cachedValueArea1 != null)
                for (int idx = 0; idx < cachedValueArea1.Length; idx++)
                    if (cachedValueArea1[idx].InclWeekendVol == inclWeekendVol && cachedValueArea1[idx].OpenHour == openHour && cachedValueArea1[idx].OpenMinute == openMinute && Math.Abs(cachedValueArea1[idx].PctOfVolumeInVA - pctOfVolumeInVA) <= double.Epsilon && cachedValueArea1[idx].PresentMethod == presentMethod && cachedValueArea1[idx].ProfileType == profileType && Math.Abs(cachedValueArea1[idx].SessionLengthInHours - sessionLengthInHours) <= double.Epsilon && cachedValueArea1[idx].TotalSlots == totalSlots && cachedValueArea1[idx].EqualsInput(input))
                        return cachedValueArea1[idx];

            dValueArea1 indicator = new dValueArea1();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.InclWeekendVol = inclWeekendVol;
            indicator.OpenHour = openHour;
            indicator.OpenMinute = openMinute;
            indicator.PctOfVolumeInVA = pctOfVolumeInVA;
            indicator.PresentMethod = presentMethod;
            indicator.ProfileType = profileType;
            indicator.SessionLengthInHours = sessionLengthInHours;
            indicator.TotalSlots = totalSlots;
            indicator.SetUp();

            dValueArea1[] tmp = new dValueArea1[cachedValueArea1 == null ? 1 : cachedValueArea1.Length + 1];
            if (cachedValueArea1 != null)
                cachedValueArea1.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cachedValueArea1 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The Value Area is the price range where 70% of yesterdays price / volume traded.  See Larry Levin www.secretsoftraders.com/ for more info
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.dValueArea1 dValueArea1(bool inclWeekendVol, int openHour, int openMinute, double pctOfVolumeInVA, int presentMethod, ValueAreaType profileType, double sessionLengthInHours, int totalSlots)
        {
            return _indicator.dValueArea1(Input, inclWeekendVol, openHour, openMinute, pctOfVolumeInVA, presentMethod, profileType, sessionLengthInHours, totalSlots);
        }

        /// <summary>
        /// The Value Area is the price range where 70% of yesterdays price / volume traded.  See Larry Levin www.secretsoftraders.com/ for more info
        /// </summary>
        /// <returns></returns>
        public Indicator.dValueArea1 dValueArea1(Data.IDataSeries input, bool inclWeekendVol, int openHour, int openMinute, double pctOfVolumeInVA, int presentMethod, ValueAreaType profileType, double sessionLengthInHours, int totalSlots)
        {
            return _indicator.dValueArea1(input, inclWeekendVol, openHour, openMinute, pctOfVolumeInVA, presentMethod, profileType, sessionLengthInHours, totalSlots);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The Value Area is the price range where 70% of yesterdays price / volume traded.  See Larry Levin www.secretsoftraders.com/ for more info
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.dValueArea1 dValueArea1(bool inclWeekendVol, int openHour, int openMinute, double pctOfVolumeInVA, int presentMethod, ValueAreaType profileType, double sessionLengthInHours, int totalSlots)
        {
            return _indicator.dValueArea1(Input, inclWeekendVol, openHour, openMinute, pctOfVolumeInVA, presentMethod, profileType, sessionLengthInHours, totalSlots);
        }

        /// <summary>
        /// The Value Area is the price range where 70% of yesterdays price / volume traded.  See Larry Levin www.secretsoftraders.com/ for more info
        /// </summary>
        /// <returns></returns>
        public Indicator.dValueArea1 dValueArea1(Data.IDataSeries input, bool inclWeekendVol, int openHour, int openMinute, double pctOfVolumeInVA, int presentMethod, ValueAreaType profileType, double sessionLengthInHours, int totalSlots)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.dValueArea1(input, inclWeekendVol, openHour, openMinute, pctOfVolumeInVA, presentMethod, profileType, sessionLengthInHours, totalSlots);
        }

    }
}
#endregion
